<?php

namespace Litepie\Plano\Policies;

use Litepie\User\Contracts\UserPolicy;
use Litepie\Plano\Models\Plano;

class PlanoPolicy
{

    /**
     * Determine if the given user can view the plano.
     *
     * @param UserPolicy $user
     * @param Plano $plano
     *
     * @return bool
     */
    public function view(UserPolicy $user, Plano $plano)
    {
        if ($user->canDo('plano.plano.view') && $user->isAdmin()) {
            return true;
        }

        return $plano->user_id == user_id() && $plano->user_type == user_type();
    }

    /**
     * Determine if the given user can create a plano.
     *
     * @param UserPolicy $user
     * @param Plano $plano
     *
     * @return bool
     */
    public function create(UserPolicy $user)
    {
        return  $user->canDo('plano.plano.create');
    }

    /**
     * Determine if the given user can update the given plano.
     *
     * @param UserPolicy $user
     * @param Plano $plano
     *
     * @return bool
     */
    public function update(UserPolicy $user, Plano $plano)
    {
        if ($user->canDo('plano.plano.edit') && $user->isAdmin()) {
            return true;
        }

        return $plano->user_id == user_id() && $plano->user_type == user_type();
    }

    /**
     * Determine if the given user can delete the given plano.
     *
     * @param UserPolicy $user
     * @param Plano $plano
     *
     * @return bool
     */
    public function destroy(UserPolicy $user, Plano $plano)
    {
        return $plano->user_id == user_id() && $plano->user_type == user_type();
    }

    /**
     * Determine if the given user can verify the given plano.
     *
     * @param UserPolicy $user
     * @param Plano $plano
     *
     * @return bool
     */
    public function verify(UserPolicy $user, Plano $plano)
    {
        if ($user->canDo('plano.plano.verify')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the given user can approve the given plano.
     *
     * @param UserPolicy $user
     * @param Plano $plano
     *
     * @return bool
     */
    public function approve(UserPolicy $user, Plano $plano)
    {
        if ($user->canDo('plano.plano.approve')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the user can perform a given action ve.
     *
     * @param [type] $user    [description]
     * @param [type] $ability [description]
     *
     * @return [type] [description]
     */
    public function before($user, $ability)
    {
        if ($user->isSuperuser()) {
            return true;
        }
    }
}
